package ca.mcgill.sable.abc.impact.visualizer.util;

import java.io.File;
import java.util.List;

import org.eclipse.ajdt.core.javaelements.AJCompilationUnit;
import org.eclipse.ajdt.core.javaelements.AJCompilationUnitManager;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.Region;
import org.eclipse.ui.texteditor.ITextEditor;

import polyglot.util.Position;
import soot.ArrayType;
import soot.BooleanType;
import soot.CharType;
import soot.DoubleType;
import soot.FloatType;
import soot.IntType;
import soot.LongType;
import soot.RefType;
import soot.ShortType;
import soot.Type;
import soot.VoidType;
import abc.weaving.aspectinfo.Aspect;
import ca.mcgill.sable.abc.impact.visualizer.actions.RunImpactAnalysisActionDelegate;

public final class ImpactPluginUtil {

	private ImpactPluginUtil() {}
	
	public static String getWorkspaceRelativePath(String absoluteFilePath) {
		String relativePath;
		String workspaceOSPath = ResourcesPlugin.getWorkspace().getRoot().getLocation().toOSString() + File.separator;
		if (absoluteFilePath.startsWith(workspaceOSPath)) {
			relativePath = absoluteFilePath.substring(workspaceOSPath.length());
		} else {
			relativePath = absoluteFilePath;
		}
		return relativePath;
	}
	
	/**
	 * Use RunImpactAnalysisActionDelegate.getCurrentProject() since Aspect's 
	 *   Position does not include the path information 
	 * @param aspect
	 * @return
	 */
	public static IFile getFileFrom(Aspect aspect) {
		return (IFile)ImpactPluginUtil.findAjType(
				RunImpactAnalysisActionDelegate.getCurrentProject(), 
				aspect.getName()).getCompilationUnit().getResource();
//		Position pos = aspect.getPosition();
//		IProject project = RunImpactAnalysisActionDelegate.getCurrentProject();
//		String [] ss = aspect.getName().split("\\.");
//		StringBuffer fb = new StringBuffer();
//		for (int i = 0; i < ss.length - 1; i++) {
//			fb.append(ss[i] + File.separator);
//		}
//		fb.append(pos.file());
//		String filename = fb.toString();
//		
//		IFile file = project.getFile(filename);
//		return file;
	}
	
	public static IFile getFileFromAbsolutePath(String absolutePath) {
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		return root.getFile(new Path(
				getWorkspaceRelativePath(absolutePath)));
	}
	
	public static IRegion positionToRegion(ITextEditor editor, Position pos) {
		IDocument document = editor.getDocumentProvider().getDocument(editor.getEditorInput());
		int startOffset = 0, endOffset = 0;
		try {
			IRegion startRegion = document.getLineInformation(pos.line() - 1);
			IRegion endRegion = document.getLineInformation(pos.endLine() - 1);
			startOffset = startRegion.getOffset() + pos.column();
			endOffset = endRegion.getOffset() + pos.endColumn();
		} catch (BadLocationException e) {
			e.printStackTrace();
		}
		
		return new Region(startOffset, endOffset - startOffset);
	}
	
	/**
	 * Find an aspect type in a project
	 * @param project the corresponding java project of the aspectj project, 
	 *        since all aspectj project is also a java project, this works
	 * @param fullyQualifiedName fully qulified name of the aspect type to be found
	 * @return the first type having the fullyQualifiedName or
	 *         null if cannot find the type
	 */
	public static IType findAjType(IProject project, String fullyQualifiedName) {

		try {
			IJavaProject jProject = JavaCore.create(project);
			List ajUnits = AJCompilationUnitManager.INSTANCE.getAJCompilationUnits(jProject);
			for (Object o : ajUnits) {
				if (o instanceof AJCompilationUnit) {
					AJCompilationUnit ajUnit = (AJCompilationUnit)o;
					for (IType type : ajUnit.getAllTypes()) {
						if (type.getFullyQualifiedName().equals(fullyQualifiedName)) {
							return type;
						}
					}
				}
			}
		} catch (CoreException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * Find a java type in a project
	 * @param project the project which should be java project
	 * @param fullyQualifiedName fully qulified name of the type to be found
	 * @return the first type having the fullyQualifiedName or
	 *         null if cannot find the type
	 */
	public static IJavaElement findJavaType(IProject project, String fullyQualifiedName) {
		try {
			IJavaProject jProject = JavaCore.create(project);
			IType jType = jProject.findType(fullyQualifiedName);
			return jType;
		} catch (JavaModelException e) {
			e.printStackTrace();
		} catch (Exception e) { //prevent exceptions ceasing the following adpater 
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * Try to get the corresponding JDT type representation of soot type, but
	 * generic types are not considered
	 * @see org.eclipse.jdt.core.Signature 
	 * @param type
	 * @return
	 */
	public static String toJDTType (Type type) {
		String ret = type.toString();
		
		if (type instanceof IntType) {
			ret = "I";
		} else if (type instanceof CharType) {
			ret = "C";
		} else if (type instanceof DoubleType) {
			ret = "D";
		} else if (type instanceof FloatType) {
			ret = "F";
		} else if (type instanceof LongType) {
			ret = "J";
		} else if (type instanceof ShortType) {
			ret = "S";
		} else if (type instanceof VoidType) {
			ret = "V";
		} else if (type instanceof BooleanType) {
			ret = "Z";
		} else if (type instanceof ArrayType) {
			ArrayType at = (ArrayType) type;
			StringBuffer retb = new StringBuffer();
			for (int i = 0; i < at.numDimensions; i++) {
				retb.append('[');
			}
			retb.append(toJDTType(at.baseType));
			ret = retb.toString();
		} else if (type instanceof RefType) {
			RefType rt = (RefType)type;
			String cname = rt.getClassName();
			ret = "Q" + cname.substring(cname.lastIndexOf('.')+1) + ";";
		}
		
		return ret;
	}
}
